#include <QCoreApplication>
//#include <QDebug>
#include <QSslSocket>
#include <QSslCipher>
#include <QDateTime>
#include <QSslConfiguration>
#include <QTextStream>

#include "connector.h"

struct ConnectorPrivate
{
    QString host;
    int port;
    QSslSocket *sock;
};

Connector::Connector( const QString &host, int port )
{
    d = new ConnectorPrivate;
    d->host = host;
    d->port = port;
    d->sock = 0;
}

Connector::~Connector()
{
    delete d;
}

void Connector::connectToHost()
{
    QTextStream cout(stdout);
    cout << "Connecting..." << endl << endl;

    QSslConfiguration configSsl = QSslConfiguration::defaultConfiguration();
    configSsl.setProtocol(QSsl::AnyProtocol);

    d->sock = new QSslSocket(this);
    d->sock->setSslConfiguration(configSsl);
    connect( d->sock, SIGNAL(encrypted()), this, SLOT(ready()) );
    connect( d->sock, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError(QAbstractSocket::SocketError)) );
    connect( d->sock, SIGNAL(sslErrors(const QList<QSslError> &)), this, SLOT(sslError(const QList<QSslError> &)) );

    d->sock->connectToHostEncrypted( d->host, d->port );
}

void Connector::dumpCertificate( const QSslCertificate &cert )
{
    QTextStream cout(stdout);
    cout << cert.toPem() << endl;

    cout << "== Subject Info ==" << endl;
    cout << "CommonName:\t\t" << cert.subjectInfo( QSslCertificate::CommonName ) << endl;
    cout << "Organization:\t\t" << cert.subjectInfo( QSslCertificate::Organization ) << endl;
    cout << "LocalityName:\t\t" << cert.subjectInfo( QSslCertificate::LocalityName ) << endl;
    cout << "OrganizationalUnitName:\t" << cert.subjectInfo( QSslCertificate::OrganizationalUnitName ) << endl;
    cout << "StateOrProvinceName:\t" << cert.subjectInfo( QSslCertificate::StateOrProvinceName ) << endl;

    QMultiMap<QSsl::AlternateNameEntryType, QString> altNames = cert.alternateSubjectNames();
    if ( !altNames.isEmpty() ) {
        cout << "Alternate Subject Names (DNS):" << endl;
        foreach (const QString &altName, altNames.values(QSsl::DnsEntry)) {
            cout << altName << endl;
        }

        cout << "Alternate Subject Names (Email):" << endl;
        foreach (const QString &altName, altNames.values(QSsl::EmailEntry)) {
            cout << altName << endl;
        }
    }

    cout << "\n== Issuer Info ==" << endl;
    cout << "CommonName:\t\t" << cert.issuerInfo( QSslCertificate::CommonName ) << endl;
    cout << "Organization:\t\t" << cert.issuerInfo( QSslCertificate::Organization ) << endl;
    cout << "LocalityName:\t\t" << cert.issuerInfo( QSslCertificate::LocalityName ) << endl;
    cout << "OrganizationalUnitName:\t" << cert.issuerInfo( QSslCertificate::OrganizationalUnitName ) << endl;
    cout << "StateOrProvinceName:\t" << cert.issuerInfo( QSslCertificate::StateOrProvinceName ) << endl;

    cout << "\n== Certificate ==" << endl;
    //cout << "Serial Number:\t\t" << cert.serialNumber() << endl; // This seems buggy
    cout << "Effective Date:\t\t" << cert.effectiveDate().toString() << endl;
    cout << "Expiry Date:\t\t" << cert.expiryDate().toString() << endl;
    cout << "Valid:\t\t\t" << (cert.isValid() ? "Yes" : "No") << endl;
}

void Connector::dumpCipher( const QSslCipher &cipher )
{
    QTextStream cout(stdout);
    cout << "\n== Cipher ==" << endl;

    cout << "Authentication:\t\t" << cipher.authenticationMethod() << endl;
    cout << "Encryption:\t\t" << cipher.encryptionMethod() << endl;
    cout << "Key Exchange:\t\t" << cipher.keyExchangeMethod() << endl;
    cout << "Cipher Name:\t\t" << cipher.name() << endl;
    cout << "Protocol:\t\t" <<  cipher.protocolString() << endl;
    cout << "Supported Bits:\t\t" << cipher.supportedBits() << endl;
    cout << "Used Bits:\t\t" << cipher.usedBits() << endl;
}

void Connector::ready()
{
    QTextStream cout(stdout);
    cout << " === Peer Certificate ===" << endl;

    QSslCertificate cert = d->sock->peerCertificate();
    dumpCertificate( cert );

    QSslCipher cipher = d->sock->sessionCipher();
    dumpCipher( cipher );

    cout << endl << "Done" << endl;
    qApp->quit();
}

void Connector::socketError( QAbstractSocket::SocketError error )
{
    QTextStream cout(stdout);
    cout << "Socket Error: " << d->sock->errorString() << " (" << int(error) << ")" << endl;
}

void Connector::sslError( const QList<QSslError> &errors )
{
    QTextStream cout(stdout);
    foreach( const QSslError &error, errors ) {
        cout << "SSL Error: " << error.errorString() << endl;
    }

    // This is only used because we are interested in dumping all the info
    // do NOT use if you want real security (it introduces man-in-the-middle
    // attacks).
    d->sock->ignoreSslErrors();
}
